####################################################################################
# Copyright (C) 2019                                                               #
# TETCOS, Bangalore. India                                                         #
#                                                                                  #
# Tetcos owns the intellectual property rights in the Product and its content.     #
# The copying, redistribution, reselling or publication of any or all of the       #
# Product or its content without express prior written consent of Tetcos is        #
# prohibited. Ownership and / or any other right relating to the software and all  #
# intellectual property rights therein shall remain at all times with Tetcos.      #
#                                                                                  #
# Author:    Dilip Kumar Kundrapu                                                  #
# Date:      16-01-2019                                                            #
#                                                                                  #
# ---------------------------------------------------------------------------------#


# This is the script for the Most Forward within Fixed Radius R (MFR) alforithm

from math import sqrt #for using sqrt() in MFR
from collections import OrderedDict #for making the dictionary as ordered

#-----------------------Functions------------------------
def _distance(a,b):
    return sqrt((b[0]-a[0])*(b[0]-a[0])+(b[1]-a[1])*(b[1]-a[1]))

def _area(a,b,c):
    return abs(  ( (a[0]*(b[1]-c[1])) + (b[0]*(c[1]-a[1])) + (c[0]*(a[1]-b[1])) ) / 2  )

def _height(a,b):
    return ((2 * a)/b)

def _projDist(a,c):
    return sqrt((c*c)-(a*a))


#-----------------------File I/P-------------------------
nodes = OrderedDict()
with open('device_log.txt','r') as f:
    for lineno, line in enumerate(f, 1):
        line = line.strip()
        if not line:
            continue
        try:
            node_name, xpos, ypos = line.split()
            nodes[node_name] = [float(xpos), float(ypos)]
        except Exception as e:
            print("line #{} seems broken (got {})".format(lineno, e))


#-----------------Declarations for MFR-------------------
source = 'WIRELESS_NODE_1'
dest = 'WIRELESS_NODE_6'

tx = 110.00 #transmission range
temp=0

temp_node = source

route = list()
route.append(source)


#-------------------The MFR protocol---------------------
for i in range(len(list(nodes))):

    #to only do the computation if the current node is a route node.
    if list(nodes)[i] != temp_node:
        continue

    base = _distance(nodes[list(nodes)[i]],nodes[dest])

    #If the current node is dest. node, then don't go into the next 'for' loop.
    if list(nodes)[i] == dest:
        break

    for j in range(i+1,len(list(nodes))):
        result = _distance(nodes[list(nodes)[i]],nodes[list(nodes)[j]])

        if result <= tx:
            area = _area (nodes[list(nodes)[i]],nodes[list(nodes)[j]],nodes[dest])
            height = _height(area,base)
            projDist = _projDist(height,result)

            #to compare the distances and select the most forward node along the line from current node to destination
            if projDist > temp:
                temp = projDist
                temp_node = list(nodes)[j]

    if temp_node not in route:
        route.append(temp_node)
    temp = 0

print (route)
